.. _extending:

Extending ``bidict``
--------------------

Although bidict ships with all the bidict types we just covered,
it's always possible users might need something more than what's provided.
For this reason,
bidict was written with extensibility in mind.

Let's look at some examples.

.. include:: overwritingbidict.rst.inc

.. include:: sortedbidicts.rst.inc
