.. See also "__credits__" in ../bidict/__init__.py

- Thanks to Gregory Ewing for the name.

- Thanks to Terry Reedy for suggesting the slice syntax
  (it was fun while it lasted).

- Thanks to Raymond Hettinger for suggesting namedbidict
  and pointing out various caveats.

- Thanks to Francis Carr for the idea of storing the inverse bidict.

- Thanks to Brianna Laugher and Adopt Pytest Month for choosing bidict
  as one of Adopt Pytest Month 2015's selected projects.

- Thanks to Tom Viner for all the help as bidict's mentor
  during Adopt Pytest Month 2015.

- Thanks to the Pytest team and to David MacIver for the great testing tools.

- Thanks to Daniel Pope for various suggestions enhancing bidict's Python zen.

- Thanks to David Turner for the generous code review.

- Thanks to Michael Arntzenius for the valuable design discussion and feedback.

- Thanks to Jozef Knaperek for the bugfix.

- Thanks to @muhammadfaisal40 for the
  `CC BY 3.0 <https://creativecommons.org/licenses/by/3.0/us/>`_
  `icon <https://thenounproject.com/term/book/1330481/>`_
  that bidict's logo was derived from.
